@@ -366,71 +366,6 @@ class JointLMDataset(BaseDataset):
         }
 
 
-class MaskedLMDataset(BaseDataset):
-    def __init__(
-            self,
-            texts: Union[list[str], HfDataset],
-            tokenizer: PreTrainedTokenizer,
-            max_seq_len: int = 1024,
-            mask_prob: float = 0.15,
-            hf_field: str = 'text',
-            *args,
-            **kwargs
-    ):
-        super(MaskedLMDataset, self).__init__(texts, tokenizer, max_seq_len, hf_field, *args, **kwargs)
-        self.mask_prob = mask_prob
-
-    def __getitem__(self, idx: int) -> dict[str, torch.Tensor]:
-        inputs = self.get_tokenized_text(idx)
-
-        input_ids = inputs['input_ids'][0]
-        if self.is_pre_tokenized:
-            input_ids = input_ids.clone()
-        attention_mask = inputs['attention_mask'][0]
-        labels = input_ids.clone()
-
-        # Create masked indices
-        masked_indices = torch.bernoulli(
-            torch.full(labels.shape, self.mask_prob)
-        ).bool() & attention_mask.bool()
-
-        # Apply mask
-        labels[~masked_indices] = -100
-        input_ids[masked_indices] = self.tokenizer.mask_token_id
-
-        return {
-            'input_ids': input_ids,
-            'attention_mask': attention_mask,
-            'labels': labels
-        }
-
-
-class AutoregressiveLMDataset(BaseDataset):
-    def __init__(
-            self,
-            texts: Union[list[str], HfDataset],
-            tokenizer: PreTrainedTokenizer,
-            max_seq_len: int = 1024,
-            hf_field: str = 'text',
-            *args,
-            **kwargs
-    ):
-        super(AutoregressiveLMDataset, self).__init__(texts, tokenizer, max_seq_len, hf_field, *args, **kwargs)
-
-    def __getitem__(self, idx: int) -> dict[str, torch.Tensor]:
-        inputs = self.get_tokenized_text(idx)
-
-        input_ids = inputs['input_ids'][0]
-        attention_mask = inputs['attention_mask'][0]
-        targets = input_ids.clone()
-
-        return {
-            'input_ids': input_ids,
-            'attention_mask': attention_mask,
-            'targets': targets
-        }
-
-
 class BaseInteractionDataset(Dataset):
     def __init__(
             self,
